package com.jzlife.report.controller;


import com.alibaba.fastjson.JSONObject;
import com.google.common.collect.Lists;
import com.jzlife.report.anno.LogRecordFiled;
import com.jzlife.report.dao.douyin.DouyinRebuyMapper;
import com.jzlife.report.dao.kaocf.KaocfMapper;
import com.jzlife.report.dto.EchartsLine;
import com.jzlife.report.dto.ReportOldCustomerRebuyDetail;
import com.jzlife.report.dto.douyin.DouyinBoardLineVo;
import com.jzlife.report.dto.douyin.DouyinRebuySingleDetailVo;
import com.jzlife.report.dto.kaocf.CityServiceQualityDto;
import com.jzlife.report.dto.kaocf.HwLineDataQueryDto;
import com.jzlife.report.dto.kaocf.HwMonthGmv;
import com.jzlife.report.dto.kaocf.KaoCom;
import com.jzlife.report.service.redis.RedisService;
import com.jzlife.report.util.*;
import com.jzlife.report.util.page.BeanUtil;
import com.jzlife.report.util.page.PagedResult;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.apache.commons.lang3.time.DateFormatUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;
import java.math.BigDecimal;
import java.time.Duration;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.*;
import java.util.stream.Collectors;

/**
 * 抖音私域复购相关报表
 */
@RestController
@RequestMapping("huawangRebuy")
public class HuawangRebuyController {

    @Resource
    private DouyinRebuyMapper rebuyMapper;

    @Autowired
    private KaocfMapper kaocfMapper;

    @Autowired
    private RedisService redisService;

    @ApiOperation(value = "花王私域复购相关报表", httpMethod = "GET", notes = "花王散单服务复购明细")
    @GetMapping("single")
    @LogRecordFiled(funName = "花王散单服务复购明细")
    public Object getSingle(
            @RequestParam(required = true) String city,
            @RequestParam(required = true) String startTime,
            @RequestParam(required = true) String endTime,
            @RequestParam(required = false) String workerName,
            @RequestParam(required = false) String rebuyStatus,
            @RequestParam(defaultValue = "1") Integer pageIndex,
            @RequestParam(defaultValue = "50") Integer pageSize,
            @RequestParam(defaultValue = "false") Boolean isExport,
            HttpServletResponse response) throws Exception {
        if (StringUtils.isBlank(startTime)) {
            return ResponseUtil.fail("服务日期不能为空");
        }

        if (isExport) {
            pageIndex = 1;
            pageSize = 100000;
        }

        List<DouyinRebuySingleDetailVo> list = rebuyMapper.getHuawangRebuySingleDetai(startTime, endTime, city, workerName, rebuyStatus, pageIndex, pageSize);
        if (isExport == false) {
            Integer total = list.size() > 0 ? list.get(0).getRowTotal() : 0;
            PagedResult<DouyinRebuySingleDetailVo> pagedResult = BeanUtil.toPagedResult(list);
            pagedResult.setTotal(total);
            pagedResult.setPageNo(pageIndex);
            pagedResult.setPageSize(pageSize);
            pagedResult.setStartPage(1);
            pagedResult.setEndPage((total / pageSize) + 1);
            pagedResult.setPages(pagedResult.getEndPage());
            return ResponseUtil.ok(pagedResult);
        } else {
            ExcelUtils.exportExcel(list, null, "Sheet1", DouyinRebuySingleDetailVo.class, "花王散单新客首单服务明细.xls", response);
            return null;
        }
    }


    @ApiOperation(value = "花王报表月gmv数据", httpMethod = "GET", notes = "花王报表月gmv数据")
    @GetMapping("getHwMonthGmv")
    @LogRecordFiled(funName = "花王报表月gmv数据")
    public Object getHwMonthGmv() throws Exception {
        String redisKey="HwMonthGmv";
        String value = redisService.get(redisKey);
        if (StringUtils.isNotEmpty(value)) {
            return ResponseUtil.ok(JSONObject.parseObject(value));
        }else {
            JSONObject result = new JSONObject();
            String monthFirstDay = DateUtils.getMonthFirstDay();
            String monthEndDay = DateUtils.getMonthEndDay();
            HwMonthGmv monthGmvs = kaocfMapper.getMonthGmvAndCount(monthFirstDay, monthEndDay);

            //同比数据(当月去年)
            String getdtqndrq = DateUtils.getdtqndrq(new Date());
            String getqndrq = DateUtils.getqndrq(monthEndDay);
            BigDecimal dyQnMonthGmv = kaocfMapper.getMonthGmv(getdtqndrq, getqndrq);
            Map<String, Object> tbData = getTbData(monthGmvs.getSumAmount(), dyQnMonthGmv);
            monthGmvs.setTbzb(tbData.get("zb").toString());
            monthGmvs.setTbzbType(tbData.get("type").toString());
            result.put("dyData", monthGmvs);

            //上月数据
            String sydyt = DateUtils.getSydyt(new Date());
            String syzhyt = DateUtils.getSyzhyt(new Date());
            HwMonthGmv syMonthGmvs = kaocfMapper.getMonthGmvAndCount(sydyt, syzhyt);


            //同比数据(上月去年)
            String syQnStartTime = DateUtils.getqndrq(sydyt);
            String syQnEndTime = DateUtils.getqndrq(syzhyt);
            BigDecimal syQnMonthGmv = kaocfMapper.getMonthGmv(syQnStartTime, syQnEndTime);
            Map<String, Object> tbData2 = getTbData(syMonthGmvs.getSumAmount(), syQnMonthGmv);
            syMonthGmvs.setTbzb(tbData2.get("zb").toString());
            syMonthGmvs.setTbzbType(tbData2.get("type").toString());
            result.put("syData", syMonthGmvs);

            //核销退款数量（同比本月去年）
            HwMonthGmv hxTkCount = kaocfMapper.getHxTkCount(getdtqndrq, getqndrq);

            //本月核销率
            BigDecimal hxl = getLv(monthGmvs.getSumCount(), monthGmvs.getHxCount());
            result.put("hxl", hxl);
            //同比核销率
            BigDecimal qnByhxl = getLv(hxTkCount.getSumCount(), hxTkCount.getHxCount());
            Map<String, Object> hxlTbData = getTbData(hxl, qnByhxl);
            result.put("hxlTb", hxlTbData.get("zb"));
            result.put("hxlTbType", hxlTbData.get("type"));

            //本月退款率
            BigDecimal tkl = getLv(monthGmvs.getSumCount(), monthGmvs.getTkCount());
            result.put("tkl", tkl);
            //同比退款率
            BigDecimal qnBytkl = getLv(hxTkCount.getSumCount(), hxTkCount.getTkCount());
            Map<String, Object> tklTbData = getTbData(tkl, qnBytkl);
            result.put("tklTb", tklTbData.get("zb"));
            result.put("tklTbType", tklTbData.get("type"));

            //履约数量（今年本月）
            HwMonthGmv lyCount = kaocfMapper.getLyCount(monthFirstDay, monthEndDay);
            BigDecimal lyy = getLv(lyCount.getSumCount(), lyCount.getLyCount());
            result.put("lyy", lyy);
            //履约数量（同比本月去年）
            HwMonthGmv qNlyCount = kaocfMapper.getLyCount(getdtqndrq, getqndrq);
            BigDecimal qnBylyy = getLv(qNlyCount.getSumCount(), qNlyCount.getLyCount());
            Map<String, Object> lyyTbData = getTbData(lyy, qnBylyy);
            result.put("lyyTb", lyyTbData.get("zb"));
            result.put("lyyTbType", lyyTbData.get("type"));

            //复购率
            result.put("fgl", "0");
            result.put("fglTb", 100);
            result.put("fglTbType", 1);

            //当月完成工单数
            Integer dyWcCount = kaocfMapper.getLyWcCount(monthFirstDay, monthEndDay);
            //去年完成工单数
            Integer quDyWcCount = kaocfMapper.getLyWcCount(monthFirstDay, monthEndDay);

            //差评率(本月差评数/本月履约工单数)
            Integer cpCount = kaocfMapper.getCpCount(monthFirstDay, monthEndDay);
            BigDecimal cpl = getLv(dyWcCount, cpCount);
            result.put("cpl", cpl);
            //同比差评率
            Integer qnCpCount = kaocfMapper.getCpCount(getdtqndrq, getqndrq);
            BigDecimal qnCpl = getLv(quDyWcCount, qnCpCount);
            Map<String, Object> cplTbData = getTbData(cpl, qnCpl);
            result.put("cplTb", cplTbData.get("zb"));
            result.put("cplTbType", cplTbData.get("type"));


            //本月好评率(本月好评数/本月履约工单数)
            HwMonthGmv hpCount = kaocfMapper.getHpCount(monthFirstDay, monthEndDay);
            BigDecimal hpl = getLv(dyWcCount, hpCount.getHpCount());
            result.put("hpl", hpl);
            //同比
            HwMonthGmv quHpCount = kaocfMapper.getHpCount(getdtqndrq, getqndrq);
            BigDecimal qnHpl = getLv(quDyWcCount, quHpCount.getHpCount());
            Map<String, Object> hplTbData = getTbData(hpl, qnHpl);
            result.put("hplTb", hplTbData.get("zb"));
            result.put("hplTbType", hplTbData.get("type"));

            Duration duration = Duration.between(LocalDateTime.now(), LocalDateTime.now().plusMinutes(60));
            redisService.set(redisKey, JSONObject.toJSONString(result), duration);

            return ResponseUtil.ok(result);
        }
    }

    @ApiOperation(value = "花王看板相关报表", httpMethod = "POST", notes = "花王近30天销量及服务情况")
    @PostMapping("lineData")
    @LogRecordFiled(funName = "花王近30天销量及服务情况")
    public Object lineData(@RequestBody HwLineDataQueryDto dto,HttpServletResponse response) throws Exception {

        dto.setChannel(dto.getChannel()!=null ?dto.getChannel():112);
        dto.setCityId(dto.getCityId()!=null?dto.getCityId():0);  //全部城市
        //定义4条曲线
        EchartsLine echOrdersList = new EchartsLine("line");//订单量
        echOrdersList.setName("销量");
        echOrdersList.setData(Lists.newArrayList());

        EchartsLine echRefundList = new EchartsLine("line");//退单量
        echRefundList.setName("退单量");
        echRefundList.setData(Lists.newArrayList());

        EchartsLine echServicedList = new EchartsLine("line");//已服务量
        echServicedList.setName("已服务订单");
        echServicedList.setData(Lists.newArrayList());

        EchartsLine echunbookedList = new EchartsLine("line");//未预约
        echunbookedList.setName("未预约订单");
        echunbookedList.setData(Lists.newArrayList());

        EchartsLine matchingList = new EchartsLine("line");//匹配中
        matchingList.setName("匹配中");
        matchingList.setData(Lists.newArrayList());

       /* EchartsLine dGmv = new EchartsLine("bar");//直营
        dGmv.setName("直营GMV");
        dGmv.setStack("Total2");
        dGmv.setData(Lists.newArrayList());*/

        EchartsLine gmv = new EchartsLine("bar");//gmv
        gmv.setName("GMV");
        gmv.setStack("Total");
        gmv.setData(Lists.newArrayList());

        EchartsLine sgmv = new EchartsLine("bar");//gmv
        sgmv.setName("已服务GMV");
        sgmv.setStack("Total3");
        sgmv.setData(Lists.newArrayList());

        String redisKey="";
        if(dto.getDateType()==0) {
             redisKey = "HwBoardLineVo" + dto.getChannel() + "_" + dto.getStartDate() + "_" + dto.getEndDate() + "_" + dto.getCityId();
        }else if (dto.getDateType()==1){
             redisKey = "HwBoardLineVo" + dto.getChannel() + "_" + dto.getMonthStartDate() + "_" + dto.getMonthEndDate() + "_" + dto.getCityId();
        }
        List<DouyinBoardLineVo> list;
        String value = redisService.get(redisKey);
        if (StringUtils.isNotEmpty(value)) {
            list = JSONObject.parseArray(value, DouyinBoardLineVo.class);
            if (dto.getIsExport() != null && dto.getIsExport()) {
                ExcelUtils.exportExcel(list, null, "Sheet1", DouyinBoardLineVo.class, "订单销售及履约情况.xls", response);
                return null;
            }
        }else {
            dto.setCityId(dto.getCityId()==0?null:dto.getCityId());
            list = kaocfMapper.getLineData(dto);
            Duration duration = Duration.between(LocalDateTime.now(), LocalDateTime.now().plusMinutes(60));
            redisService.set(redisKey, JSONObject.toJSONString(list), duration);
            if(dto.getIsExport() !=null && dto.getIsExport()){
                ExcelUtils.exportExcel(list, null, "Sheet1", DouyinBoardLineVo.class, "订单销售及履约情况.xls", response);
                return null;
            }
        }

        List<String> dayList = Lists.newArrayList();
        if(dto.getDateType()==0){
            //天
            DateTimeFormatter fmt = DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss");
            Duration duration = Duration.between(LocalDateTime.parse(dto.getStartDate()+" 00:00:00", fmt), LocalDateTime.parse(dto.getEndDate()+" 23:59:59", fmt));
            //天日期格式化
            int days =(int) duration.toDays()+1;
            for (int i = days; i > 0; i--) {
                String date = DateUtils.format.format(DateUtil.addDay(DateUtil.stringToDate(dto.getEndDate()), -i));
                dayList.add(date);

                List<DouyinBoardLineVo> dList = list.stream().filter(item -> item.getDate().equals(date)).collect(Collectors.toList());
                echOrdersList.getData().add(dList.size() > 0 ? dList.get(0).getTotal() : 0);
                echRefundList.getData().add(dList.size() > 0 ? dList.get(0).getRefunds() : 0);
                echServicedList.getData().add(dList.size() > 0 ? dList.get(0).getServiced() : 0);
                echunbookedList.getData().add(dList.size() > 0 ? dList.get(0).getUnbooked() : 0);
                matchingList.getData().add(dList.size() > 0 ? dList.get(0).getMatching() : 0);
                gmv.getData().add(dList.size() > 0 ? dList.get(0).getGmv() : 0);
                sgmv.getData().add(dList.size() > 0 ? dList.get(0).getServicedGMV() : 0);
            }
        }else if(dto.getDateType()==1){
            //月
            String startDate = dto.getMonthStartDate();
            String endDate = dto.getMonthEndDate();
            List<String> months = AccountDate.getAppointMonths(startDate, endDate);
            for (String month : months) {
                dayList.add(month);
                List<DouyinBoardLineVo> dList = list.stream().filter(item -> item.getDate().equals(month)).collect(Collectors.toList());
                echOrdersList.getData().add(dList.size() > 0 ? dList.get(0).getTotal() : 0);
                echRefundList.getData().add(dList.size() > 0 ? dList.get(0).getRefunds() : 0);
                echServicedList.getData().add(dList.size() > 0 ? dList.get(0).getServiced() : 0);
                echunbookedList.getData().add(dList.size() > 0 ? dList.get(0).getUnbooked() : 0);
                matchingList.getData().add(dList.size() > 0 ? dList.get(0).getMatching() : 0);
                gmv.getData().add(dList.size() > 0 ? dList.get(0).getGmv() : 0);
                sgmv.getData().add(dList.size() > 0 ? dList.get(0).getServicedGMV() : 0);

            }

        }
        List<EchartsLine> echLiss = Lists.newArrayList();
        echLiss.add(echOrdersList);
        echLiss.add(echRefundList);
        echLiss.add(echServicedList);
        echLiss.add(echunbookedList);
        echLiss.add(gmv);
        echLiss.add(sgmv);

        Map<String, Object> data = new HashMap<String, Object>();
        data.put("series", echLiss);
        data.put("days", dayList);

        return ResponseUtil.ok(data);
    }

    @ApiOperation(value = "花王看板相关报表", httpMethod = "POST", notes = "花王服务质量汇总")
    @PostMapping("hwCityServiceQuality")
    @LogRecordFiled(funName = "花王服务质量汇总")
    public Object hwCityServiceQuality(@RequestBody HwLineDataQueryDto dto,HttpServletResponse response) throws Exception {

        String redisKey="";
        if(dto.getDateType()==0) {
            redisKey = "HwCityServiceQuality_" + dto.getStartDate() + "_" + dto.getEndDate();
        }else if (dto.getDateType()==1){
            redisKey = "HwCityServiceQuality_" + dto.getMonthStartDate() + "_" + dto.getMonthEndDate();
        }
        if(StringUtils.isNotEmpty(dto.getChannelName())){
            redisKey="_"+dto.getChannelName();
        }
        String value = redisService.get(redisKey);   //非导出缓存

        if(dto.getIsExport() !=null && dto.getIsExport()){
            //导出缓存
            String exportValue = redisService.get(redisKey+"_export");
            if(StringUtils.isNotEmpty(exportValue)){
                List<CityServiceQualityDto> cityServiceQualityDtos = JSONObject.parseArray(exportValue, CityServiceQualityDto.class);
                ExcelUtils.exportExcel(cityServiceQualityDtos, null, "Sheet1", CityServiceQualityDto.class, "花王服务质量汇总.xls", response);
                return null;
            }
        }
        if (StringUtils.isNotEmpty(value)){
            return ResponseUtil.ok(JSONObject.parseObject(value));
        }else {
            //定义柱状图
            EchartsLine hfbmy = new EchartsLine("bar");//gmv
            hfbmy.setName("回访不满意");
            hfbmy.setStack("Total");
            hfbmy.setData(Lists.newArrayList());

            EchartsLine ksl = new EchartsLine("bar");//gmv
            ksl.setName("客诉量");
            ksl.setStack("Total2");
            ksl.setData(Lists.newArrayList());

            //定义曲线
            EchartsLine echKsList = new EchartsLine("line");//订单量
            echKsList.setName("客诉率");
            echKsList.setData(Lists.newArrayList());


            EchartsLine hpl = new EchartsLine("bar");//gmv
            hpl.setName("好评量");
            hpl.setStack("Total3");
            hpl.setData(Lists.newArrayList());

            EchartsLine echHplList = new EchartsLine("line");//退单量
            echHplList.setName("好评率");
            echHplList.setData(Lists.newArrayList());

            //城市完成工单数
            List<CityServiceQualityDto> cityWcGds = kaocfMapper.getCityWcGds(dto);
            //城市客诉量
            List<CityServiceQualityDto> cityKs = kaocfMapper.getCityKs(dto);
            //城市好评量
            List<CityServiceQualityDto> cityHp = kaocfMapper.getCityHp(dto);
            //回访不满意数量
            List<CityServiceQualityDto> cityHfbmy = kaocfMapper.getCityHfbmy(dto);

            List<CityServiceQualityDto> exportData = new ArrayList<>();

            List<String> cityList = Arrays.asList("上海", "杭州", "北京", "成都", "深圳", "苏州", "南京", "广州", "重庆", "长沙", "西安", "青岛", "昆山");
            for (String city : cityList) {

                List<CityServiceQualityDto> wcsList = cityWcGds.stream().filter(item -> item.getCity().equals(city)).collect(Collectors.toList());
                List<CityServiceQualityDto> ksList = cityKs.stream().filter(item -> item.getCity().equals(city)).collect(Collectors.toList());
                List<CityServiceQualityDto> hpList = cityHp.stream().filter(item -> item.getCity().equals(city)).collect(Collectors.toList());
                List<CityServiceQualityDto> hfbmyList = cityHfbmy.stream().filter(item -> item.getCity().equals(city)).collect(Collectors.toList());
                Integer kslNum = ksList.size() > 0 ? ksList.get(0).getKsCount() : 0;
                BigDecimal kslBfb = ksList.size() > 0 ? (wcsList.size() > 0 ? getLv(wcsList.get(0).getWcCount(), ksList.get(0).getKsCount()) : new BigDecimal(100)) : new BigDecimal(0);
                Integer hplNum = hpList.size() > 0 ? hpList.get(0).getHpCount() : 0;
                BigDecimal hplBfb = hpList.size() > 0 ? (wcsList.size() > 0 ? getLv(wcsList.get(0).getWcCount(), hpList.get(0).getHpCount()) : new BigDecimal(100)) : new BigDecimal(0);
                Integer hfbmyNum = hfbmyList.size() > 0 ? hfbmyList.get(0).getHfbmyCount() : 0;


                CityServiceQualityDto cityServiceQualityDto = new CityServiceQualityDto();
                cityServiceQualityDto.setCity(city);
                cityServiceQualityDto.setHfbmyCount(hfbmyNum);
                cityServiceQualityDto.setKsCount(kslNum);
                cityServiceQualityDto.setKsl(kslBfb.toString());
                cityServiceQualityDto.setHpCount(hplNum);
                cityServiceQualityDto.setHpl(hplBfb.toString());
                exportData.add(cityServiceQualityDto);

                ksl.getData().add(kslNum);
                echKsList.getData().add(kslBfb);
                hpl.getData().add(hplNum);
                echHplList.getData().add(hplBfb);
                hfbmy.getData().add(hfbmyNum);


            }
            if (dto.getIsExport() !=null && dto.getIsExport()) {
                ExcelUtils.exportExcel(exportData, null, "Sheet1", CityServiceQualityDto.class, "花王服务质量汇总.xls", response);
                return null;
            } else {
                List<EchartsLine> echLiss = Lists.newArrayList();
                echLiss.add(hfbmy);
                echLiss.add(ksl);
                echLiss.add(echKsList);
                echLiss.add(hpl);
                echLiss.add(echHplList);

                Map<String, Object> data = new HashMap<String, Object>();
                data.put("series", echLiss);
                data.put("cityList", cityList);
                Duration duration = Duration.between(LocalDateTime.now(), LocalDateTime.now().plusMinutes(60));
                redisService.set(redisKey, JSONObject.toJSONString(data), duration); //网页数据缓存
                redisService.set(redisKey+"_export", JSONObject.toJSONString(exportData), duration); //导出数据缓存
                return ResponseUtil.ok(data);
            }
        }
    }


    private Map<String,Object> getTbData(BigDecimal money1, BigDecimal money2){
        Map<String, Object> resultMap = new HashMap<>();
        int result2 = money1.compareTo(money2);
        if (result2<0) {
            //下降
            BigDecimal zb = getTb(money2,money1);
            resultMap.put("zb",zb);
            resultMap.put("type",0);
        }else if(result2>0) {
            //上升
            BigDecimal zb = getTb(money1,money2);
            resultMap.put("zb",zb);
            resultMap.put("type",1);
        } else {
            resultMap.put("zb",0);
            resultMap.put("type",0);
        }
        return resultMap;
    }

    private BigDecimal getTb(BigDecimal money1,BigDecimal money2){
        return money1.subtract(money2).divide(money1,2, BigDecimal.ROUND_HALF_UP).multiply(new BigDecimal("100"));
    }

    /**
     * 算达成率
     * @return
     */
    private BigDecimal getLv(Integer sumCount,Integer count){
        if(sumCount==0 && count==0){
            return new BigDecimal("0");
        }
        if(sumCount==0){
            return new BigDecimal("100");
        }
        if(count==0){
            return new BigDecimal("0");
        }
        return new BigDecimal(count).divide(new BigDecimal(sumCount),2, BigDecimal.ROUND_HALF_UP).multiply(new BigDecimal("100"));

    }
}


